"""Define a base object for interacting with 17track.net."""

import requests

from py17track.exceptions import HTTPError

API_BASE = 'https://t.17track.net/restapi'


class BaseAPI(object):
    """Define a class that represents an API request."""

    def __init__(self):
        """Initialize."""
        self.full_url = None
        self.session = requests.Session()

    def request(self, method_type, url, **kwargs):
        """Define a generic request."""
        self.full_url = '{0}/{1}'.format(API_BASE, url)
        method = getattr(self.session, method_type)
        resp = method(self.full_url, **kwargs)

        # I don't think it's good form to make end users of py17track have to
        # explicitly catch exceptions from a sub-library, so here, I wrap the
        # Requests HTTPError in my own:
        try:
            resp.raise_for_status()
        except requests.exceptions.HTTPError as exc_info:
            raise HTTPError(str(exc_info)) from None

        return resp

    def post(self, url, **kwargs):
        """Define a generic POST request."""
        return self.request('post', url, **kwargs)



"""Define an 17track.com client."""

from py17track.api import BaseAPI
from py17track.exceptions import InvalidTrackingNumberError
from py17track.package import Package


class Client(BaseAPI):  # pylint: disable=too-few-public-methods
    """Define a 17track.net client."""

    def track(self, *tracking_numbers: str) -> dict:
        """Get tracking info for one or more tracking numbers."""
        data = {'data': [{'num': num} for num in tracking_numbers]}
        resp = self.post('track', json=data).json()

        if not resp.get('dat'):
            raise InvalidTrackingNumberError('Invalid data')

        packages = {}
        for info in resp['dat']:
            package = Package(info)
            packages[package.tracking_number] = package
        return packages
